"""
Quick visualisation routines for ksn analysis and knickpoint extraction anaysis.
B.G
"""
from matplotlib import pyplot as plt
import numpy as np
import pandas as pd
import os
import lsdtopytools.quickplot_utilities as QUT, lsdtopytools.quickplot as QP
from lsdtopytools import LSDDEM


def plot_ksn_map(mydem ,figure_width = 4, figure_width_units = "inches", cmap = "RdBu_r", hillshade = True, 
	alpha_hillshade = 0.45, color_min = None, color_max = None, dpi = 300, output = "save",
	 format_figure = "png", min_point_size = 0.5, max_point_size = 3, ksn_min = None, ksn_max = None, ksn_colormod = "default", size_outline = 1, fontsize_ticks =6, fontsize_label = 8):
	"""
		Plot ksn on hillshade or black background. Obviously, you need to calculate it beforeend!
		Arguments:
			mydem: the LSDDEM object. Required.
			figure_width (float): figure width in inches (Default value because of matplotlib...) or centimeters.
			figure_width_units (str): "inches" (Default) or "centimetres"
			cmap (str): the name of the colormap to use, see https://matplotlib.org/examples/color/colormaps_reference.html for list.
			hillshade (bool): Drape an hillshade on the top of the figure
			alpha_hillshade (float): regulate the transparency of the hillshade. Between 0 (transparent) and 1 (opaque).
			color_min (float): min elevation corresponding to the min color on the colormap. None = auto.
			color_max (float): see color_min and replace min by max.
			dpi (int): Figure quality, increase will increase the figure quality and its size ofc.
			output (str): "save" to save figure, "return" to return fig/axis, "show" to show.
			format_figure (str): the extension of the saved bitmap: "png", "jpg", "svg" reccomended. For the rest see: https://matplotlib.org/api/_as_gen/matplotlib.pyplot.savefig.html
			min_point_size (float): minimum size for the point scaled by Drainage area.
			max_point_size (float): maximum size for the point scaled by Drainage area.
			ksn_min (float): k_sn value that defines the minimum value of the color bar (all the ksn below that value will have the minimum color). Can be percentile or absolute value.
			ksn_max (float): k_sn value that defines the maximum value of the color bar (all the ksn above that value will have the maximum color). Can be percentile or absolute value.
			ksn_colormod (str): "default" to scale colormap per value, "percentile" tp scale using stats.
			size_outline (float): size of the basin outline. 0 for no outline. hacky but simple innit?
			fontsize_ticks (float): Font size of the ticks. Might be a bit random, matplotlib can be difficult to tame.
			fontsize_label (float): Font size of the labels. Might be a bit random, matplotlib can be difficult to tame.
		Returns:
			Depends on the output parameters
		Authors:
			B.G
		Date:
			23/02/2019
	"""
	if(hillshade):
		fig, ax1 = QP.plot_nice_topography(mydem ,figure_width = figure_width, figure_width_units = figure_width_units, hillshade = hillshade, 
	alpha_hillshade = 1, color_min = None, color_max = None, dpi = 300, output = "return")
	else:
		# First creating the figure and axis
		fig = QUT.create_single_fig_from_width(mydem.ncols, mydem.nrows, width = figure_width, width_units = figure_width_units, background_color = "white")
		gs = plt.GridSpec(100,100,bottom=0.15, left=0.20, right=0.98, top=0.98)
		ax1 = fig.add_subplot(gs[0:100,0:100], facecolor = "none")
		ax1.imshow(mydem.cppdem.get_PP_raster(), mydem.extent, cmap = "gray", vmin = 10000000, vmax = 10000001, zorder = 1, fontsize_ticks =fontsize_ticks, fontsize_label = fontsize_ticks) # Forcing black background
	
	# Getting basin outlines
	QUT.add_basin_outlines(mydem, fig, ax1, size_outline = size_outline)

	# colour bounds if not manually chosen
	if(ksn_colormod.lower() == "default"):
		ksn_min = mydem.df_ksn["m_chi"].min() if(ksn_min is None) else ksn_min
		ksn_max = mydem.df_ksn["m_chi"].max() if(ksn_max is None) else ksn_max
	if(ksn_colormod.lower() == "percentile"):
		ksn_min = mydem.df_ksn["m_chi"].quantile(0.10) if(ksn_min is None) else mydem.df_ksn["m_chi"].quantile(ksn_min)
		ksn_max = mydem.df_ksn["m_chi"].quantile(0.90) if(ksn_max is None) else mydem.df_ksn["m_chi"].quantile(ksn_max)

	# Plotting ksn
	cb = ax1.scatter(mydem.df_ksn["x"], mydem.df_ksn["y"], s= QUT.size_my_points(mydem.df_ksn["drainage_area"].values, min_point_size, max_point_size),
		c=mydem.df_ksn["m_chi"].values, cmap = cmap, vmin = ksn_min, vmax = ksn_max, zorder = 5, lw = 0) # done

	# Colorbar
	cax = plt.colorbar(cb)
	cax.ax.tick_params(labelsize = fontsize_ticks)
	cax.ax.set_ylabel(r"$k_{sn}$", fontsize = fontsize_label, rotation = -270)


	#Done?
	return QUT.finalise_fig(fig, ax1, output, mydem, "ksn_map", format_figure, dpi)


"""
Controls the TribBas Model. Highly eperimental and evolves really rapidly So no doc so far sns.
No documentation so far, B.G. is still testing it.
"""
from lsdtt_xtensor_python import run_TribBas_to_steady_state, prebuild_TribBas, run_TribBas # Old stuff
from lsdtt_xtensor_python import LSDTribBas_cpp
import numba
import numpy as np
import pandas as pd
from lsdtopytools import raster_loader as rl, geoconvtools as gc
from lsdtopytools.lsdtopytools_utilities import save_to_database, load_from_database, load_metadata_from_database
from . import LSDDEM


class LSDTribBas(object):
    """
        The LSDTribBas class provides a high level interface to the LSDTribBas Model in LSDTopoTools.
        May disappear at some points but I need that for automation.
        Will develop if needed.
        Boris
    """

    def __init__(self, raster_name = "buzau.bil", sources_method = "threshold", threshold_contributing_pixels = 1000, source_csv = "buzau_Wsources.csv", output_name = "Clegg.h5", outlet_x=0, outlet_y =0, search_node_radius = 100, target_node = 20, MC_iterations = 50, skip = 1, sigma =2, m = 1.4, n = 3.11, A0 = 1, verbose = True, already_preprocessed = True):
        """
        Constructor

        """
        # First: Do you want the model to talk much?
        self.verbose = verbose

        #Loading raster
        print("Loading raster into the system") if self.verbose else 0
        A = rl.load_raster(raster_name)
        print("Raster loaded !") if self.verbose else 0
        if(~already_preprocessed):
            B = LSDDEM(file_name = raster_name)
            B.PreProcessing()
            A["array"] = B.cppdem.get_PP_raster()

        print("Loading my sources") if self.verbose else 0
        # I will need the sources
        ## Either I read it from a csv
        if("sources_method" == "csv"):
            sources = pd.read_csv(source_csv)
        else:
            # Or I recalculate it from threshold
            B.ExtractRiverNetwork( method = "area_threshold", area_threshold_min = threshold_contributing_pixels)
            sources = B.cppdem.get_sources_full()
            for key,val in sources.items():
                print("key: %s shape: %s"%(key, val.shape))
            # sources["node"] = sources["nodeID"]
            sources = pd.DataFrame({"x": sources["X"], "y":sources["Y"],"node":sources["nodeID"].astype(np.int32)})
            # sources = sources.rename(columns={"nodeID": "node", "X": "x", "Y": "y"})
            print(sources)


        # Trimming my sources as they are extracted from a larger raster
        SOX = sources['x'][(sources['x'] > A['x_min']) & (sources['x']<A['x_max'])].values
        SOY = sources['y'][(sources['y'] > A['y_min']) & (sources['y']<A['y_max'])].values
        print("Sources ready to be ingested") if self.verbose else 0

        print(SOX)


        # Prebuilding the model
        self.n = n
        self.m = m
        self.A0 = A0

        # HD5 file
        self.hd5_name = output_name
        self.run_default_ID = 0

        print("Now prebuilding your model: \n-> getting all the flow informations\n-> ingesting the sources to extract a river network\n-> Calculate initial ksn using Mudd et al., 2014 (JGR)\n-> Order and format my nodes\nIt can take a bit of time...") if self.verbose else 0
        self.TBcpp = LSDTribBas_cpp(A["nrows"], A["ncols"], A["x_min"], A["y_min"], A["res"], A["nodata"][0], A["array"],SOX, SOY, m, n, A0, outlet_x, outlet_y, search_node_radius, target_node, MC_iterations, skip, 10, sigma)
        print("Model constructed!") if self.verbose else 0
        self.n_node = self.TBcpp.m_chi().shape[0]

        self.base_param = [A["nrows"], A["ncols"], A["x_min"], A["y_min"], A["res"], A["nodata"][0], A["array"],SOX, SOY, m, n, A0, outlet_x, outlet_y, search_node_radius, target_node, MC_iterations, skip, 10, sigma]

        print("Saving the initial state to hd5...")
        storer = pd.HDFStore(self.hd5_name)
        storer.put("initial",pd.DataFrame({"flow_distance": self.TBcpp.flow_distance(), "elevation": self.TBcpp.elevation()}),format = "t")
        storer.close()

        # self.K_to_SS = {}

    def run_model(self,K_field = 5e-8, uplift_field = 0.001, tolerance_p = 0.95, tolerance_delta = 0.5, timestep = 1000, final_tol = 0.95, save_dir = "./", n_timestep = "auto", save_step = 10, min_n_timestep = 25 
        , max_n_timestep = 10000, run_save_ID = None):
        """
            Just run the model
            K and U fields can be scalar or ndarray
        """

        # getting the uplift and K to the right format
        if(~isinstance(uplift_field,np.ndarray)):
            uplift_field = np.zeros(self.n_node) + uplift_field
        if(~isinstance(K_field,np.ndarray)):
            K_field = np.zeros(self.n_node) + K_field

        if(isinstance(n_timestep,str)):
            if(n_timestep == "auto"):
                results = self.TBcpp.run_model(timestep, save_step,tolerance_p, tolerance_delta, min_n_timestep , uplift_field, K_field, max_n_timestep)
        else:
            results = self.TBcpp.run_model(timestep, save_step,tolerance_p, tolerance_delta, n_timestep, uplift_field, K_field, n_timestep)

        if(run_save_ID is None):
            outID = "run_" + str(self.run_default_ID + 1)
            print("Defaulting the name to %s" %(outID))
        else:
            outID = str(run_save_ID)

        storer = pd.HDFStore(self.hd5_name)
        storer.put(outID,pd.DataFrame(results), format="t")
        storer.close()


    def m_and_ns(self, min_n = 1, min_m = 1, step = 0.1, nsteps = 40, min_K = 1e-9, range_of_K = [1e-9,1e-8,1e-7,1e-6,1e-5,1e-4,1e-3], uplift_field = 0.001, tolerance_p = 0.95, tolerance_delta = 0.5, timestep = 1000, 
        final_tol = 0.95, save_dir = "./", min_n_timestep = 25 , max_n_timestep = 10000, run_save_ID = None, interval_of_test = 1000):
        """
        try stuff on best fit m and n
        """    
        from lsdtt_xtensor_python import get_median_profile

        print("Initialising the model, it will take a while as I am calculating m_chi for all m/n combination")
        # initialising the ranges
        range_m = []
        range_n = []
        m_chi_in_theta = {} # Dictionnary to avoid duplicity of data
        chi_in_theta = {}
        compmet = {}

        this_U = uplift_field
        unique_ID_m_and_ns = 0
        # Output managers
        analysis_names = []# 

        # Getting the ranges of m n and m/n
        for i in range(nsteps):
            range_m.append(min_m + step * i)
            range_n.append(min_n + step * i)
        

        for tn in range_n:
            for tm in range_m:
                if(tm<tn):
                    ratio = tm/tn
                    ratio = str(ratio)[:4]
                    if(~(ratio in m_chi_in_theta)):
                        # print("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",ratio, tm, tn)
                        temp_OB = LSDTribBas_cpp(self.base_param[0], self.base_param[1], self.base_param[2], self.base_param[3], self.base_param[4], self.base_param[5],self.base_param[6],self.base_param[7], self.base_param[8], range_m[-1] , range_n[-1], self.base_param[11], self.base_param[12], self.base_param[13], self.base_param[14], self.base_param[15], self.base_param[16], self.base_param[17], self.base_param[18], self.base_param[19])
                        m_chi_in_theta[ratio] = temp_OB.m_chi()
                        chi_in_theta[ratio] = temp_OB.chi()

        # for key,val in m_chi_in_theta.items():
        #     print(key)

        print("Model initialised!")

        save_to_database(self.hd5_name,"reference_mchi", pd.DataFrame(m_chi_in_theta))


        print("Ok Let's roll")
        this_K = min_K
        for this_K in (range_of_K):
            # getting the uplift and K to the right format
            if(~isinstance(uplift_field,np.ndarray)):
                uplift_field = np.zeros(self.n_node) + uplift_field

            K_field = np.zeros(self.n_node) + this_K

            for m in range_m:
                for n in range_n:
                    if(m<n): # Checking that m/n < 1
                        print("Calculating m: %s and n: %s" %(m,n))
                        self.TBcpp.set_m_and_n(m,n)
                        # Running the model to steady state
                        results = pd.DataFrame(self.TBcpp.run_model(timestep, 100000,tolerance_p, tolerance_delta, min_n_timestep , uplift_field, K_field, max_n_timestep))
                        maxi = 0
                        for col in results.columns :
                            if( float(col) > maxi):
                                maxi = col
                        # Getting the results
                        this_elev_array = results[maxi].values
                        temp_df = pd.DataFrame({"flow_distance": self.TBcpp.flow_distance(), "initial_elevation": self.TBcpp.elevation(), "SS_elevation": this_elev_array})
                        temp_df = temp_df.sort_values("flow_distance")

                        # Getting the median values
                        compa = get_median_profile(temp_df["flow_distance"].values, temp_df["SS_elevation"].values - temp_df["initial_elevation"].values,interval_of_test, 1)
                        # saving the values
                        kalak = str(m) + "_" + str(n)
                        compmet[kalak + "_elev"] = compa["Y"]

                        # dealing with median m_chi
                        this_ratio = str(m/n)[:4]
                        this_mchi = self.TBcpp.first_order_m_chi_from_custarray(chi_in_theta[this_ratio],this_elev_array)
                        compa_2 = get_median_profile(temp_df["flow_distance"].values, m_chi_in_theta[this_ratio] - this_mchi, interval_of_test, 1)
                        compmet[kalak + "_mchi"] = compa_2["Y"]



            compmet["X"] = compa["X"] #  for plotting. Because flow distance is always the same, X is as well
            df = pd.DataFrame(compmet)
            # temp for testing

            # df.to_csv("test_of_mn.csv", index = False)
            analysis_names.append("mn_test_" + str(unique_ID_m_and_ns))
            save_to_database(self.hd5_name,"mn_test_" + str(unique_ID_m_and_ns), df, metadata = {"uplift_field": this_U, "K_field": this_K})
            # storer = pd.HDFStore(self.hd5_name)
            # storer.put("U%s_K%s"%(this_U,this_K),df, format="t")
            # storer.close()
            unique_ID_m_and_ns += 1
            # this_K = 10 ** (np.log10(this_K) + 0.1)
        # Saving lists of m,n,k and U that have been effectively tested
        save_to_database(self.hd5_name,"mn_test_metadata", pd.DataFrame({"test": [0,0,0,1,2]}), metadata = {"analysis_names": analysis_names})



    def TODO(self):
        print("""
            TODO:
            - Adapt the run_model's output to get the stat of fit for each timestep
            - implement m_chi calculation and comparison
            - Simplification/multithreading
            - Dorce K or U to behave by segments
            """)





class Analytic_TribBas(object):
    """
        This class build and provides analytic solutions for solving the lithology vs uplift problem. It uses the equation of k_sn calculated with M_chi.
    """
    
    def __init__(self, m = 1.08, n=3.11, min_K = -10, max_K = -4, min_U = 0.0001, max_U = 0.005, step_K = 0.01 , step_U = 0.00001, K_in_log_space = True):


        super(Analytic_TribBas, self).__init__()
        self.m = m
        self.n = n
        self.K_in_log_space = True        

        ######################## Generate 3D function
        # Need here to get the ranges of possibilities for U and K
        self.range_K = []
        tKsat = min_K
        if(K_in_log_space):
            while(tKsat <= max_K):
                # tKsat += 0.01 * 10**(math.floor(math.log10(tKsat))) not in log space
                self.range_K.append(tKsat)
                tKsat += step_K

        else:
            print("TODO: not in log space (you probably don't want to do that though)")
            quit()


        self.range_uplift = []
        step_U = min_U
        while(step_U <= max_U):
            self.range_uplift.append(step_U)
            step_U += 0.00001


        self.range_K = np.array(self.range_K)
        self.range_uplift = np.array(self.range_uplift)



    def get_U_from_mchi_and_K(self, this_K, this_m_chi, logged = False):
        if(logged):
            this_K = 10**this_K
        return this_K * np.exp(self.n * np.log(this_m_chi))

    def get_K_from_mchi_and_U(self, this_uplift, this_m_chi):
        return this_uplift / np.exp(self.n * np.log(this_m_chi))


    def generate_2D_state_of_equilibrium_from_U_and_m_chi(self, range_of_U, range_of_mchi):
        return get_K_from_mchi_and_U(range_of_U[:,np.newaxis],range_of_mchi[np.newaxis,:])









































































































































































































################## OLD ROUTINES I USED FOR CHRISTMAS. BASIC AND BRUTFORCE

    # def hunt_my_K(self,initial_k = 5e-5, reference_k_step = 1e-5, uplift_field = 0.001, min_iteration_per_run = 20, tolerance_p = 0.95, tolerance_delta = 0.5, timestep = 1000, final_tol = 0.95,  final_tol_delta = 0.5, save_dir = "./", max_iterations_per_test= 500, max_global_iterations = 100 ):
    #     """
    #         Iterative method to force K refining
    #     """
    #     # We force an uplift field to remain constant
    #     U = np.zeros(self.prebuilt["elevation"].shape[0]) + uplift_field
    #     # Initition of erodibility
    #     K = np.zeros(self.prebuilt["elevation"].shape[0]) + initial_k
    #     # Initialization of adaptative erodibility step
    #     refK = np.zeros(self.prebuilt["elevation"].shape[0]) + reference_k_step
    #     # sign of K last time
    #     signk = np.zeros(self.prebuilt["elevation"].shape[0])
    #     last_sign_k = np.zeros(self.prebuilt["elevation"].shape[0])
    #     how_many_time_unchanged = np.zeros(self.prebuilt["elevation"].shape[0])


    #     run = {}

    #     catched = False

    #     cpt = 0
    #     this_prebuilt = self.prebuilt.copy()

    #     bdf = pd.DataFrame(data = self.prebuilt)
    #     bdf.to_csv(save_dir+"base.csv", index = False)



    #     print("Chasing my K!")
    #     while(catched == False and cpt<max_global_iterations): # safety to 100
    #         cpt+=1 # ᶘ °㉨°ᶅ
    #         print("MODEL RUN %s"%(cpt), end="\r")

    #         # Run little K
    #         results = run_TribBas_to_steady_state(self.m,self.n, timestep, 10000, tolerance_p, tolerance_delta, 20, U, K, this_prebuilt, max_iterations_per_test)

    #         # print("FOUND STEADY-STATE, REFINING MY K")

    #         # get the latest result
    #         testk = 0
    #         maxk = 0
    #         for key,val in results.items():
    #             testk = int(key) # string to int
    #             print(key)
    #             if(testk>maxk):
    #                 maxk = testk
    #         maxk = str(int(maxk)) # getting the right format
    #         maxk = maxk.split(".")[0] # Making sure no decimal are there
    #         print("DEBUG::maxk:", maxk)
    #         new_elev = results[maxk]
    #         base_elev = np.copy(self.prebuilt["elevation"])
    #         print("DEBUG::mean_Baseelev:", np.mean(base_elev))

    #         n_tol = new_elev[np.abs(new_elev-base_elev)<final_tol_delta].shape[0]
    #         n_tot = new_elev.shape[0]
    #         if(n_tol/n_tot>final_tol):
    #             catched = True
    #             print("I THINK I FOUND MY K !!!")
    #         else:
    #             print("REFINING MY K ...")
    #             signk[new_elev<base_elev] = -1
    #             signk[new_elev>base_elev] = 1
    #             signk[np.abs(new_elev-base_elev)<final_tol_delta] = 0

    #             if(cpt==1): # first run is 1 (incrementation at the beginning of the loop, see ᶘ °㉨°ᶅ)
    #                 last_sign_k = np.copy(signk)

    #             K, refK, how_many_time_unchanged = self.adjust_my_K_wild(base_elev,new_elev,refK,signk,last_sign_k,final_tol,K,how_many_time_unchanged)

    #             print("I ADAPTED %s/%s K."%(signk[signk!=0].shape[0],signk.shape[0]))

    #         df = pd.DataFrame(data = {"K":K, "elevation": new_elev})

    #         name = str(cpt)
    #         while(len(name)<4):
    #             name = "0" +name

    #         df.to_csv(save_dir + "attempt_%s_K.csv"%(name))
    #         this_prebuilt["elevation"] = new_elev
    #         last_sign_k = np.copy(signk)
    #     return K



    # # Temp function, will move to cpp
    # @numba.jit()
    # def adjust_my_K_wild(self, base_elev,new_elev,refK,signk,last_sign_k,final_tol,K,how_many_time_unchanged):

    #     for i in range(base_elev.shape[0]):
    #         # Need adjustment?
    #         if(abs(new_elev[i]-base_elev[i])>final_tol):
    #             if(signk[i] != last_sign_k[i]):
    #                 refK[i] = refK[i]/2
    #                 how_many_time_unchanged[i] = 0
    #             else:
    #                 how_many_time_unchanged[i] = how_many_time_unchanged[i]+1
    #                 if(how_many_time_unchanged[i]>10):
    #                     refK[i] = refK[i]*1.5
    #                     how_many_time_unchanged[i] = 0



    #             last_K = K[i]
    #             K[i] += signk[i]*refK[i]
    #             if(K[i]<=0):
    #                 K[i] = refK[i]/2 # HACKY WAY HERE
    #                 refK[i] = refK[i]/2 # HACKY WAY HERE


    #     return K, refK,how_many_time_unchanged

    # def hunt_my_U(self,initial_uplift = 0.001, reference_uplift_step = 0.0001, min_iteration_per_run = 20 , K_field = 0.001, tolerance_p = 0.95, tolerance_delta = 0.5, timestep = 1000, final_tol = 0.95,  final_tol_delta = 0.5, save_dir = "./", max_iterations_per_test= 500, max_global_iterations = 100 ):
    #     """
    #         Iterative method to force K refining
    #     """
    #     # We force an uplift field to remain constant
    #     U = np.zeros(self.prebuilt["elevation"].shape[0]) + initial_uplift
    #     # Initition of erodibility
    #     K = np.zeros(self.prebuilt["elevation"].shape[0]) + K_field
    #     # Initialization of adaptative erodibility step
    #     refU = np.zeros(self.prebuilt["elevation"].shape[0]) + reference_uplift_step
    #     # sign of K last time
    #     signU = np.zeros(self.prebuilt["elevation"].shape[0])
    #     last_sign_k = np.zeros(self.prebuilt["elevation"].shape[0])
    #     how_many_time_unchanged = np.zeros(self.prebuilt["elevation"].shape[0])


    #     run = {}

    #     catched = False

    #     cpt = 0
    #     this_prebuilt = self.prebuilt.copy()

    #     bdf = pd.DataFrame(data = self.prebuilt)
    #     bdf.to_csv(save_dir+"base.csv", index = False)



    #     print("Chasing my theoretical Uplift field!")
    #     while(catched == False and cpt<max_global_iterations): # safety to 100
    #         cpt+=1 # ᶘ °㉨°ᶅ
    #         print("MODEL RUN %s"%(cpt))

    #         # Run little K
    #         results = run_TribBas_to_steady_state(self.m,self.n, timestep, 10000, tolerance_p, tolerance_delta, min_iteration_per_run, U, K, this_prebuilt, max_iterations_per_test)

    #         print("FOUND STEADY-STATE, REFINING MY K")

    #         # get the latest result
    #         testU = 0
    #         maxU = 0
    #         for key,val in results.items():
    #             testU = int(key) # string to int
    #             print(key)
    #             if(testU>maxU):
    #                 maxU = testU
    #         maxU = str(int(maxU)) # getting the right format
    #         maxU = maxU.split(".")[0] # Making sure no decimal are there
    #         # print("DEBUG::maxU:", maxU)
    #         new_elev = results[maxU]
    #         base_elev = np.copy(self.prebuilt["elevation"])
    #         # print("DEBUG::mean_Baseelev:", np.mean(base_elev))

    #         n_tol = new_elev[np.abs(new_elev-base_elev)<final_tol_delta].shape[0]
    #         n_tot = new_elev.shape[0]
    #         if(n_tol/n_tot>final_tol):
    #             catched = True
    #             print("I THINK I FOUND MY U :)")
    #         else:
    #             print("REFINING MY U :)")
    #             signU[new_elev<base_elev] = 1
    #             signU[new_elev>base_elev] = -1
    #             signU[np.abs(new_elev-base_elev)<final_tol_delta] = 0

    #             if(cpt==1): # first run is 1 (incrementation at the beginning of the loop, see ᶘ °㉨°ᶅ)
    #                 last_sign_U = np.copy(signU)

    #             U, refU, how_many_time_unchanged = self.adjust_my_U_wild(base_elev,new_elev,refU,signU,last_sign_U,final_tol,U,how_many_time_unchanged)

    #             print("I ADAPTED %s/%s U."%(signU[signU!=0].shape[0],signU.shape[0]))

    #         df = pd.DataFrame(data = {"U":U, "elevation": new_elev})

    #         name = str(cpt)
    #         while(len(name)<4):
    #             name = "0" +name

    #         df.to_csv(save_dir + "attempt_%s_U.csv"%(name))
    #         this_prebuilt["elevation"] = new_elev
    #         last_sign_U = np.copy(signU)

    #     return U



    # # Temp function, will move to cpp
    # @numba.jit()
    # def adjust_my_U_wild(self, base_elev,new_elev,refU,signU,last_sign_U,final_tol,U,how_many_time_unchanged):

    #     for i in range(base_elev.shape[0]):
    #         # Need adjustment?
    #         if(abs(new_elev[i]-base_elev[i])>final_tol):
    #             if(signU[i] != last_sign_U[i]):
    #                 refU[i] = refU[i]/2
    #                 how_many_time_unchanged[i] = 0
    #             else:
    #                 how_many_time_unchanged[i] = how_many_time_unchanged[i]+1
    #                 if(how_many_time_unchanged[i]>10):
    #                     refU[i] = refU[i]*1.5
    #                     how_many_time_unchanged[i] = 0



    #             last_U = U[i]
    #             U[i] += signU[i]*refU[i]
    #             if(U[i]<=0):
    #                 U[i] = refU[i]/2 # HACUY WAY HERE
    #                 refU[i] = refU[i]/2 # HACUY WAY HERE


    #     return U, refU,how_many_time_unchanged















#!/usr/bin/env python
"""
Command-line tool to control the concavity constraining tools
Mudd et al., 2018
So far mostly testing purposes
B.G.
"""
import matplotlib;matplotlib.use("Agg")
from lsdtopytools import LSDDEM # I am telling python I will need this module to run.
from lsdtopytools import argparser_debug as AGPD # I am telling python I will need this module to run.
from lsdtopytools import quickplot as qp, quickplot_movern as qmn # We will need the plotting routines
import time as clock # Basic benchmarking
import sys # manage the argv
import numpy as np
from matplotlib import pyplot as plt

def hypsometry_tools_general():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["hypsometry"] = False
	default_param["absolute_elevation"] = False
	default_param["normalise_to_outlets"] = False
	default_param["X"] = None
	default_param["Y"] = None

	default_param = AGPD.ingest_param(default_param, sys.argv)

	# Checking the param
	if(isinstance(default_param["X"],str)):
		X = [float(default_param["X"])]
		Y = [float(default_param["Y"])]
	else:
		X = [float(i) for i in default_param["X"]]
		Y = [float(i) for i in default_param["Y"]]


	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool run concavity analysis tools from LSDTopoTools.
			Description of the algorithms in Mudd et al., 2018 -> https://www.earth-surf-dynam.net/6/505/2018/
			To use, run the script with relevant options, for example:
				lsdtt-concavity-tools.py file=myraster.tif quick_movern X=5432 Y=78546

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				hypsometry: run hypsometric calculations
				absolute_elevation: hypsometric elevation will be the absolute value rather than the relavie/normalised one
				X: X coordinate of the outlet (So far only single basin is supported as this is an alpha tool)
				Y: Y coordinate of the outlet (So far only single basin is supported as this is an alpha tool)
				help: if written, diplay this message. Documentation soon to be written.
			""")
		quit()


	print("Welcome to the command-line tool to plot basin-wide hypsometry stuff!")
	print("Let me first load the raster ...")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = False, verbose = False)
	print("Got it. Now dealing with the depressions ...")
	mydem.PreProcessing(filling = True, carving = True, minimum_slope_for_filling = 0.0001) # Unecessary if already preprocessed of course.
	print("Done! Extracting the river network")
	mydem.ExtractRiverNetwork( method = "area_threshold", area_threshold_min = 1000)
	print("I have some rivers for you! Defining the watershed of interest...")


	mydem.DefineCatchment( method="from_XY", X_coords = X, Y_coords = Y, test_edges = False, coord_search_radius_nodes = 25, coord_threshold_stream_order = 1)
	print("I should have it now.")
	print("I got all the common info, now I am running what you want!")
	mydem.GenerateChi()

	print("Producing a figure of your catchement just for you to check if their location is OK!")
	qp.plot_check_catchments(mydem)


	if default_param["hypsometry"]:
		print("I am going to plot relative hypsometric curves for your different basins")
		print("This is quite quickly coded (for Jorien) and will probably evolve in the future!")
		
		fig,ax = plt.subplots()

		# Basin Array
		BAS = mydem.cppdem.get_chi_basin()
		TOPT = mydem.cppdem.get_PP_raster()
		DAD = mydem.cppdem.get_DA_raster()


		for bas in np.unique(BAS):
			if(bas != -9999):
				# getting hte topo raster ONLY for current basin
				this_topo = TOPT[BAS == bas].ravel()
				# Now getting the DA
				this_DA = DAD[BAS == bas].ravel()
				A = np.nanmax(this_DA) # maximum area of that basin

				# Normalising the topo
				if(default_param["absolute_elevation"] == False):
					this_topo = (this_topo - np.nanmin(this_topo))
					this_topo = this_topo/np.nanmax(this_topo)
				elif(default_param["normalise_to_outlets"]):
					this_topo = (this_topo - np.nanmin(this_topo))

				# Y is h/H, X is a/A
				Y =[];X=[]
				for i in range(101):
					if(default_param["absolute_elevation"] == False):
						Y.append(i*0.01)
					else:
						Y.append(np.percentile( this_topo,i))

					X.append(np.nanmax(this_DA[this_topo>=Y[-1]])/A)
				ax.plot(X,Y,lw = 1, label = bas, alpha = 0.8)

		ax.set_xlabel(r"$\frac{a}{A}$")
		if(default_param["absolute_elevation"] == False):
			ax.set_ylabel(r"$\frac{h}{H}$")
		else:
			ax.set_ylabel("Elevation (m)")

		ax.legend()

		if(default_param["absolute_elevation"] == False):
			suffix ="_relative_hypsometry"
		else:
			suffix = "_absolute_hypsometry"
			if(default_param["normalise_to_outlets"]):
				suffix +="_norm2outlet"

		plt.savefig(mydem.save_dir+mydem.prefix+suffix+"."+"png", dpi = 500)
		
		plt.clf()




	print("Finished!")




#!/usr/bin/env python
"""
Command-line tool to control the concavity constraining tools
Mudd et al., 2018
So far mostly testing purposes
B.G.
"""
from lsdtopytools import LSDDEM # I am telling python I will need this module to run.
from lsdtopytools import argparser_debug as AGPD # I am telling python I will need this module to run.
from lsdtopytools import quickplot as qp, quickplot_movern as qmn # We will need the plotting routines
import time as clock # Basic benchmarking
import sys # manage the argv
import pandas as pd
import lsdtopytools as lsd
import numpy as np
import numba as nb
import pandas as pd
from matplotlib import pyplot as plt
import sys
import matplotlib
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
import math
from lsdtopytools.numba_tools import travelling_salesman_algortihm, remove_outliers_in_drainage_divide
import random
import matplotlib.gridspec as gridspec
from multiprocessing import Pool, current_process
from scipy import spatial

class NoBasinFoundError(Exception):
	pass

def main_concavity():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["quick_movern"] = False
	default_param["X"] = None
	default_param["Y"] = None

	default_param = AGPD.ingest_param(default_param, sys.argv)


	# Checking the param
	if(isinstance(default_param["X"],str)):
		X = [float(default_param["X"])]
		Y = [float(default_param["Y"])]
	else:
		X = [float(i) for i in default_param["X"]]
		Y = [float(i) for i in default_param["Y"]]


	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool run concavity analysis tools from LSDTopoTools.
			Description of the algorithms in Mudd et al., 2018 -> https://www.earth-surf-dynam.net/6/505/2018/
			To use, run the script with relevant options, for example:
				lsdtt-concavity-tools.py file=myraster.tif quick_movern X=5432 Y=78546

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				quick_movern: run disorder metrics and plot a result figure (you jsut need to write it)
				X: X coordinate of the outlet (So far only single basin is supported as this is an alpha tool)
				Y: Y coordinate of the outlet (So far only single basin is supported as this is an alpha tool)
				help: if written, diplay this message. Documentation soon to be written.
			""")
		quit()


	print("Welcome to the command-line tool to constrain your river network concavity. Refer to Mudd et al., 2018 -> https://www.earth-surf-dynam.net/6/505/2018/ for details about these algorithms.")
	print("Let me first load the raster ...")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = False, verbose = False)
	print("Got it. Now dealing with the depressions ...")
	mydem.PreProcessing(filling = True, carving = True, minimum_slope_for_filling = 0.0001) # Unecessary if already preprocessed of course.
	print("Done! Extracting the river network")
	mydem.ExtractRiverNetwork( method = "area_threshold", area_threshold_min = 1000)



	print("I have some rivers for you! Defining the watershed of interest...")
	mydem.DefineCatchment( method="from_XY", X_coords = X, Y_coords = Y, test_edges = False, coord_search_radius_nodes = 25, coord_threshold_stream_order = 1)
	print("I should have it now.")
	print("I got all the common info, now I am running what you want!")

	if(default_param["quick_movern"]):
		print("Initialising Chi-culations (lol)")
		mydem.GenerateChi()
		print("Alright, getting the disorder metrics for each chi values. LSDTopoTools can split a lot of messages, sns.")
		mydem.cppdem.calculate_movern_disorder(0.15, 0.05, 17, 1, 1000) # start theta, delta, n, A0, threashold
		print("I am done, plotting the results now")
		qmn.plot_disorder_results(mydem, legend = False, normalise = True, cumulative_best_fit = False)
		qp.plot_check_catchments(mydem)
		qmn.plot_disorder_map(mydem, cmap = "RdBu_r")
		print("FInished with quick disorder metric")

	print("Finished!")


def temp_concavity_FFS_all():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["process_main"] = False
	default_param["X"] = None
	default_param["Y"] = None
	default_param["AT"] = None
	default_param["ATM"] = None
	default_param["n_proc"] = None
	default_param["min_DA"] = None
	default_param["max_DA"] = None
	default_param["prefix"] = ""

	prefix = default_param["prefix"]

	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			Experimental command-line tools for concavity constraining. Deprecated now.
			""")
		quit()

	X = float(default_param["X"])
	Y = float(default_param["Y"])
	min_DA = float(default_param["min_DA"])
	max_DA = float(default_param["max_DA"])


	area_threshold_main_basin = float(default_param["ATM"])
	area_threshold = float(default_param["AT"])
	n_proc = int(default_param["n_proc"])

	dem_name = default_param["file"]
	if(default_param["process_main"]):
		lsd.concavity_automator.process_main_basin(dem_name, dem_path = "./" ,already_preprocessed = True , X_outlet = X, Y_outlet = Y, area_threshold = area_threshold, area_threshold_to_keep_river = area_threshold_main_basin, prefix = default_param["prefix"])
	else:
		lsd.concavity_automator.get_all_concavity_in_range_of_DA_from_baselevel(dem_name, dem_path = "./" ,already_preprocessed = True , X_outlet = X, Y_outlet = Y, min_DA = min_DA, max_DA = max_DA, area_threshold = area_threshold, n_proc = n_proc, prefix = default_param["prefix"])
		lsd.concavity_automator.post_process_basins(prefix = default_param["prefix"])
	
	for key,val in default_param.items():
		default_param[key] = [val]

	pd.DataFrame(default_param).to_csv(prefix +"log_concavity_FFS_sstuff.txt", index = False)

def concavity_single_basin():
	"""
Command-line tool to constrain concavity for a single basin from its XY coordinates using Normalised disorder method.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
X=234 -> X Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 
Y=234 -> Y Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 
AT=5000 -> Area threshold in number of pixel to initiate a river: lower nummber <=> denser network (quickly increases the processing time)
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
mode=g -> DEFAULT is g . Processing mode: can be "g" for generating data, "p" or plotting previously generated data or "all" for everything.

Example:

lsdtt-concFFS-single file=DEM.tif already_preprocessed X=43422 Y=5353497 AT=2500 prefix=FeatherRiver mode=all
			"""
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["X"] = None
	default_param["Y"] = None
	default_param["AT"] = None
	default_param["prefix"] = ""
	default_param["mode"] = "g"
	default_param["n_tribs_by_combinations"] = 3
	# Ingesting the parameters
	default_param = AGPD.ingest_param(default_param, sys.argv)

	prefix = default_param["prefix"]

	if(default_param["help"] or len(sys.argv)==1):
		print("""
Command-line tool to constrain concavity for a single basin from its XY coordinates using Normalised disorder method.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
X=234 -> X Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 
Y=234 -> Y Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 
AT=5000 -> Area threshold in number of pixel to initiate a river: lower nummber <=> denser network (quickly increases the processing time)
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
mode=g -> DEFAULT is g . Processing mode: can be "g" for generating data, "p" or plotting previously generated data or "all" for everything.

Example:

lsdtt-concFFS-single file=DEM.tif already_preprocessed X=43422 Y=5353497 AT=2500 prefix=FeatherRiver mode=all
			""")
		quit()

	# Reformatting some values that sometimes are not formatted correctly
	X = float(default_param["X"])
	Y = float(default_param["Y"])
	area_threshold = float(default_param["AT"])
	dem_name = default_param["file"]
	default_param["n_tribs_by_combinations"] = int(default_param["n_tribs_by_combinations"])
	# Wrapper to the processing function (the convoluted ls method makes multiprocessing easier when processing several basins)
	ls = [0,X,Y,area_threshold,prefix]
	# Calling th requested codes
	if("all" in default_param["mode"].lower() or "g" in default_param["mode"].lower()):
		lsd.concavity_automator.process_basin(ls,ignore_numbering = True, overwrite_dem_name = dem_name, n_tribs_by_combo =  default_param["n_tribs_by_combinations"])
	if("all" in default_param["mode"].lower() or "p" in default_param["mode"].lower()):
		lsd.concavity_automator.plot_basin(ls,ignore_numbering = True, overwrite_dem_name = dem_name)
	
	# Saving a log of processing
	for key,val in default_param.items():
		default_param[key] = [val]
	pd.DataFrame(default_param).to_csv(prefix +"log_concavity_FFS_single_basin_sstuff.txt", index = False)


def concavity_multiple_basin():
	"""
Command-line tool to constrain concavity for a multiple basin from their XY coordinates using Normalised disorder method.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
csv=outlets.csv -> Name of the csv file containing the following columns: "X", "Y" and "area_threshold" for each basins to investigate. Can be generated automatically from lsdtt-concFFS-spawn-outlets
n_proc=4 -> DEFAULT is 1. Number of processors to use in parallel when possible.
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
mode=g -> DEFAULT is g . Processing mode: can be "g" for generating data, "p" or plotting previously generated data, "d" for plotting disorder map (WARNING takes time and memory) "all" for everything.

Example:

lsdtt-concFFS-multiple file=DEM.tif already_preprocessed csv=FeatherRiveroutlets.csv prefix=FeatherRiver mode=g
			"""
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["prefix"] = ""
	default_param["csv"] = ""
	default_param["n_proc"] = 1
	default_param["area_thershold_basin_extraction"] = 500
	default_param["precipitation_file"] = ""
	default_param["mode"] = "g"
	default_param["n_tribs_by_combinations"] = 3
	default_param = AGPD.ingest_param(default_param, sys.argv)
	prefix = default_param["prefix"]

	if(default_param["help"] or len(sys.argv)==1):
		print("""
Command-line tool to constrain concavity for a multiple basin from their XY coordinates using Normalised disorder method.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
csv=outlets.csv -> Name of the csv file containing the following columns: "X", "Y" and "area_threshold" for each basins to investigate. Can be generated automatically from lsdtt-concFFS-spawn-outlets
n_proc=4 -> DEFAULT is 1. Number of processors to use in parallel when possible.
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
mode=g -> DEFAULT is g . Processing mode: can be "g" for generating data, "p" or plotting previously generated data, "all" for everything.

Example:

lsdtt-concFFS-multiple file=DEM.tif already_preprocessed csv=FeatherRiveroutlets.csv prefix=FeatherRiver mode=g
			""")

	# Reading the csv file 
	df = pd.read_csv(default_param["csv"])

	# Reformatting stuff
	n_proc = int(default_param["n_proc"])
	default_param["n_tribs_by_combinations"] = int(default_param["n_tribs_by_combinations"])
	area_thershold_basin_extraction = float(default_param["area_thershold_basin_extraction"])
	dem_name = default_param["file"]

	if(default_param["precipitation_file"] == ""):
		precipitation = False
	else:
		precipitation = True

	# Processing options
	if("all" in default_param["mode"].lower() or "g" in default_param["mode"].lower()):
		lsd.concavity_automator.process_multiple_basins(dem_name, dem_path = "./",already_preprocessed = default_param["already_preprocessed"], 
			prefix = default_param["prefix"], X_outlets = df["X"].values, Y_outlets = df["Y"].values, n_proc = n_proc, area_threshold = df["area_threshold"].values, 
			area_thershold_basin_extraction = area_thershold_basin_extraction, n_tribs_by_combo =  default_param["n_tribs_by_combinations"],
			 use_precipitation_raster = precipitation, name_precipitation_raster = default_param["precipitation_file"])
		lsd.concavity_automator.post_process_analysis_for_Dstar(default_param["prefix"], n_proc = n_proc, base_raster_full_name = dem_name)
	
	if("z" in default_param["mode"].lower()):
		lsd.concavity_automator.post_process_analysis_for_Dstar(default_param["prefix"], n_proc = n_proc, base_raster_full_name = dem_name)
	
	if("p" in default_param["mode"].lower()):
		lsd.concavity_automator.plot_main_figures(default_param["prefix"])
		

	if("all" in default_param["mode"].lower() or "d" in default_param["mode"].lower()):
		lsd.concavity_automator.plot_multiple_basins(dem_name, dem_path = "./",already_preprocessed = default_param["already_preprocessed"], 
			prefix = default_param["prefix"], X_outlets = df["X"].values, Y_outlets = df["Y"].values, n_proc = n_proc, area_threshold = df["area_threshold"].values, 
			area_thershold_basin_extraction = area_thershold_basin_extraction, plot_Dstar = False, n_tribs_by_combo =  default_param["n_tribs_by_combinations"])
		lsd.concavity_automator.plot_Dstar_maps_for_all_concavities(default_param["prefix"], n_proc = n_proc)


	# Saving logs
	for key,val in default_param.items():
		default_param[key] = [val]

	pd.DataFrame(default_param).to_csv(prefix +"log_concavity_FFS_multiple_basin_sstuff.txt", index = False)

def spawn_XY_outlet():
	"""
Command-line tool to prechoose the basins used for other analysis. Outputs a file with outlet coordinates readable from other command-line tools and a basin perimeter csv readable by GISs to if the basins corresponds to your needs.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
test_edges -> OPTIONAL will test if the basin extracted are potentially influenced by nodata and threfore uncomplete. WARNING, will take out ANY basin potentially cut, if you know what you are doing, you can turn off.
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
method=from_range -> DEFAULT from_range: determine the method to select basin. Can be
		from_range -> select largest basins bigger than min_DA but smaller than max_DA (in m^2)
		min_area -> select largest basins bigger than min_DA
		main_basin -> select the largest basin
		Other methods to come.
min_elevation=45 -> DEFAULT 0. Ignore any basin bellow that elevation
area_threshold=3500 -> DEFAULT 5000. River network area threshold in number of pixels (part of the basin selection is based on river junctions HIGHLY sensitive to that variable).

Example:
lsdtt-concFFS-spawn-outlets file=DEM.tif already_preprocessed min_DA=1e7 max_DA=1e9 area_threshold=3500
			"""

	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["test_edges"] = False
	default_param["area_threshold"] = 5000
	default_param["method"] = "from_range"
	default_param["min_DA"] = 1e6
	default_param["max_DA"] = 1e9
	default_param["min_elevation"] = 0;
	default_param["prefix"] = "";
	default_param = AGPD.ingest_param(default_param, sys.argv)

	choice_of_method = ["min_area", "main_basin","from_range"]

	if(default_param["help"] or len(sys.argv)==1):
		print("""
Command-line tool to prechoose the basins used for other analysis. Outputs a file with outlet coordinates readable from other command-line tools and a basin perimeter csv readable by GISs to if the basins corresponds to your needs.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
test_edges -> OPTIONAL will test if the basin extracted are potentially influenced by nodata and threfore uncomplete. WARNING, will take out ANY basin potentially cut, if you know what you are doing, you can turn off.
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
method=from_range -> DEFAULT from_range: determine the method to select basin. Can be
		from_range -> select largest basins bigger than min_DA but smaller than max_DA (in m^2)
		min_area -> select largest basins bigger than min_DA
		main_basin -> select the largest basin
		Other methods to come.
min_elevation=45 -> DEFAULT 0. Ignore any basin bellow that elevation
area_threshold=3500 -> DEFAULT 5000. River network area threshold in number of pixels (part of the basin selection is based on river junctions HIGHLY sensitive to that variable).

Example:
lsdtt-concFFS-spawn-outlets file=DEM.tif already_preprocessed min_DA=1e7 max_DA=1e9 area_threshold=3500
			""")
		return 0;

	# Checks if the method requested is valid or not
	if(default_param["method"].lower() not in choice_of_method):
		print("I cannot recognise the method! Please choose from:")
		print(choice_of_method)
		return 0

	# Formatting parameters
	area_threshold = int(default_param["area_threshold"])
	min_DA = float(default_param["min_DA"])
	max_DA = float(default_param["max_DA"])
	min_elevation = float(default_param["min_elevation"])

	# Reading DEM
	mydem = LSDDEM(file_name = default_param["file"], path = default_param["path"], already_preprocessed = default_param["already_preprocessed"], remove_seas = True, sea_level = min_elevation)
	if(default_param["already_preprocessed"] == False):
		mydem.PreProcessing()
	# Extracting basins
	mydem.CommonFlowRoutines()
	print("Done with flow routines")
	mydem.ExtractRiverNetwork( method = "area_threshold", area_threshold_min = area_threshold)
	# Get the outlet coordinates of all the extracted basins
	print("Extracted rivers")
	df_outlet = mydem.DefineCatchment(  method = default_param["method"], min_area = min_DA, max_area = max_DA, test_edges = default_param["test_edges"])#, X_coords = [X_coordinates_outlets[7]], Y_coords = [Y_coordinates_outlets[7]])
	print("Extracted")

	for key,val in df_outlet.items():
		df_outlet[key] = np.array(df_outlet[key])
	# Getting the rivers
	mydem.GenerateChi(theta = 0.4, A_0 = 1)
	# Saing the rivers to csv
	mydem.df_base_river.to_csv(default_param["prefix"]+"rivers.csv", index = False)
	#Saving the outlet
	df_outlet["area_threshold"] = np.full(df_outlet["X"].shape[0],area_threshold)

	# print(df_outlet)

	pd.DataFrame(df_outlet).to_csv(default_param["prefix"]+"outlets.csv", index = False)

	# Getting the perimeter of basins
	this = mydem.cppdem.extract_perimeter_of_basins()
	df_perimeter = {"X":[],"Y":[],"Z":[],"IDs":[]}
	for key,val in this.items():
		df_perimeter["X"].append(np.array(val["X"]))
		df_perimeter["Y"].append(np.array(val["Y"]))
		df_perimeter["Z"].append(np.array(val["Z"]))
		df_perimeter["IDs"].append(np.full(np.array(val["Z"]).shape[0], key))

	for key,val in df_perimeter.items():
		df_perimeter[key] = np.concatenate(val)



	pd.DataFrame(df_perimeter).to_csv(default_param["prefix"]+"perimeters.csv", index = False)

def spawn_XY_outlet_subbasins():

	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["X"] = 0
	default_param["Y"] = 0
	default_param["area_threshold"] = 5000
	default_param["min_DA"] = 1e6
	default_param["max_DA"] = 1e9
	default_param["min_elevation"] = 0;
	default_param["prefix"] = "";
	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
Command-line tool to extract basin information about all the subbasins within a main one. Outputs a file with outlet coordinates readable from other command-line tools and a basin perimeter csv readable by GISs to if the basins corresponds to your needs.
Takes several arguments (the values after = are example values to adapt): 
file=NameOfFile.tif -> The code NEEDS the neame of the raster to process.
already_preprocessed -> OPTIONAL Tell the code your raster does not need preprocessing, otherwise carve the DEM (see lsdtt-depressions for more options)
prefix=test -> OPTIONAL Add a prefix to each outputted file (handy for automation)
min_elevation=45 -> DEFAULT 0. Ignore any basin bellow that elevation
area_threshold=3500 -> DEFAULT 5000. River network area threshold in number of pixels (part of the basin selection is based on river junctions HIGHLY sensitive to that variable).
min_DA=1e7 -> minimum drainage area to extract a subbasin
max_DA=1e9 -> maximum drainage area for a subbasin
X=234 -> X Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 
Y=234 -> Y Coordinate (in map unit) of the outlet (needs to be the exact pixel at the moment, will add a snapping option later) 

Example:
lsdtt-concFFS-spawn-outlets file=DEM.tif already_preprocessed min_DA=1e7 max_DA=1e9 area_threshold=3500
			""")
		return 0;

	area_threshold = int(default_param["area_threshold"])
	X = float(default_param["X"])
	min_DA = float(default_param["min_DA"])
	Y = float(default_param["Y"])
	max_DA = float(default_param["max_DA"])
	min_elevation = float(default_param["min_elevation"])

	mydem = LSDDEM(file_name = default_param["file"], path = default_param["path"], already_preprocessed = default_param["already_preprocessed"], remove_seas = True, sea_level = min_elevation)
	if(default_param["already_preprocessed"] == False):
		mydem.PreProcessing()
	# Extracting basins
	mydem.CommonFlowRoutines()
	mydem.ExtractRiverNetwork( method = "area_threshold", area_threshold_min = area_threshold)
	# df_outlet = mydem.DefineCatchment(  method = default_param["method"], min_area = min_DA, max_area = max_DA, test_edges = default_param["test_edges"])#, X_coords = [X_coordinates_outlets[7]], Y_coords = [Y_coordinates_outlets[7]])
	df_outlet = mydem.cppdem.calculate_outlets_min_max_draining_to_baselevel(X, Y, min_DA, max_DA,500)
	mydem.check_catchment_defined = True


	for key,val in df_outlet.items():
		df_outlet[key] = np.array(df_outlet[key])
	mydem.GenerateChi(theta = 0.4, A_0 = 1)
	mydem.df_base_river.to_csv(default_param["prefix"]+"rivers.csv", index = False)
	df_outlet["area_threshold"] = np.full(df_outlet["X"].shape[0],area_threshold)
	df_outlet = pd.DataFrame(df_outlet)
	df_outlet.to_csv(default_param["prefix"]+"outlets.csv", index = False)
	df_outlet["ID"] = np.array(list(range(df_outlet.shape[0])))	

	this = mydem.cppdem.extract_perimeter_of_basins()
	df_perimeter = {"X":[],"Y":[],"Z":[],"IDs":[]}
	for key,val in this.items():
		df_perimeter["X"].append(np.array(val["X"]))
		df_perimeter["Y"].append(np.array(val["Y"]))
		df_perimeter["Z"].append(np.array(val["Z"]))
		df_perimeter["IDs"].append(np.full(np.array(val["Z"]).shape[0], key))


	## Log from the analysis
	for key,val in df_perimeter.items():
		df_perimeter[key] = np.concatenate(val)
	pd.DataFrame(df_perimeter).to_csv(default_param["prefix"]+"perimeters.csv", index = False)

	

def temp_concavity_FFS_all_test_func(default_param):
	# Here are the different parameters and their default value fr this script
	# default_param = AGPD.get_common_default_param()
	# default_param["already_preprocessed"] = False
	# default_param["X"] = None
	# default_param["Y"] = None
	# default_param["AT"] = None
	# default_param["ATM"] = None
	# default_param["n_proc"] = None
	# default_param["min_DA"] = None
	# default_param["max_DA"] = None
	# default_param["prefix"] = ""

	# default_param = AGPD.ingest_param(default_param, sys.argv)


	X = float(default_param["X"])
	Y = float(default_param["Y"])
	min_DA = float(default_param["min_DA"])
	max_DA = float(default_param["max_DA"])


	area_threshold_main_basin = float(default_param["ATM"])
	area_threshold = float(default_param["AT"])
	n_proc = int(default_param["n_proc"])

	dem_name = default_param["file"]

	lsd.concavity_automator.get_all_concavity_in_range_of_DA_from_baselevel(dem_name, dem_path = "./" ,already_preprocessed = True , X_outlet = X, Y_outlet = Y, min_DA = min_DA, max_DA = max_DA, area_threshold = area_threshold, n_proc = n_proc, prefix = default_param["prefix"])
	lsd.concavity_automator.post_process_basins(prefix = default_param["prefix"])
	lsd.concavity_automator.process_main_basin(dem_name, dem_path = "./" ,already_preprocessed = True , X_outlet = X, Y_outlet = Y, area_threshold = area_threshold, area_threshold_to_keep_river = area_threshold_main_basin, prefix = default_param["prefix"])

	for key,val in default_param.items():
		default_param[key] = [val]

	pd.DataFrame(default_param).to_csv(default_param["prefix"] +"log_concavity_FFS_sstuff.txt", index = False)



def concavity_FFS_down_to_top():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["save_XY_array"] = False
	default_param["X_source"] = None
	default_param["Y_source"] = None
	default_param["min_elevation"] = None
	default_param["area_threshold"] = None
	default_param["flow_distance_step"] = None
	default_param["min_DA"] = None
	default_param["prefix"] = ""
	default_param = AGPD.ingest_param(default_param, sys.argv)


	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool run concavity analysis tools from LSDTopoTools.
			Description of the algorithms in Mudd et al., 2018 -> https://www.earth-surf-dynam.net/6/505/2018/
			To use, run the script with relevant options, for example:
				lsdtt-concavity-down-to-top file=myraster.tif ---

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				todo
				help: if written, diplay this message. Documentation soon to be written.
			""")
		quit()

	try:
		default_param["X_source"] = float(default_param["X_source"])
		default_param["Y_source"] = float(default_param["Y_source"])
		default_param["min_elevation"] = float(default_param["min_elevation"])
		default_param["area_threshold"] = float(default_param["area_threshold"])
		default_param["flow_distance_step"] = float(default_param["flow_distance_step"])
		default_param["min_DA"] = float(default_param["min_DA"])
	except:
		print(default_param)
		print("I struggle to understand your input parameters: make sure I can convert them to number")
		raise SystemExit ("I need to quit now")

	mydem = LSDDEM(file_name = default_param["file"], path = default_param["path"], already_preprocessed = default_param["already_preprocessed"], remove_seas = True, sea_level = default_param["min_elevation"])

	if(default_param["already_preprocessed"] == False):
		print("I am preprocessing your raster with default options, see lsdtt-depressions for extended options.")
		print("Command line tools can load already processed rasters with the keyword already_preprocessed")
		mydem.PreProcessing()

	# flwo routines
	print("Processing flow routines with d8 algorithm")
	mydem.CommonFlowRoutines()

	print("Extracting the river")
	river = mydem.ExtractSingleRiverFromSource(default_param["X_source"],default_param["Y_source"])
	river = pd.DataFrame(river)
	print("Saving teh river to csv file: river_for_concavity.csv")
	river.to_csv("%sriver_for_concavity.csv"%(default_param["prefix"]), index = False)

	print("Initialising the concavity analysis")
	river.sort_values("elevation", inplace = True)
	river.reset_index(drop = True, inplace = True)

	# OUtputs
	global_results = {}
	outlet_info = {"X":[], "Y":[], "elevation":[], "flow_distance":[], "drainage_area":[]}

	# potential saving of basins
	XY_basins = {}

	flow_distance_this = river["flow_distance"].min()
	index_this = river["flow_distance"].idxmin()
	while(flow_distance_this < river["flow_distance"].max() and river["drainage_area"][index_this] > default_param["min_DA"]):
		print("processing  flow distance =", flow_distance_this)
		del mydem
		mydem = LSDDEM(file_name = "%sdem_for_concavity.tif"%(default_param["prefix"]), already_preprocessed = True, remove_seas =True, sea_level = river["elevation"][index_this]-5)
		mydem.CommonFlowRoutines()
		# mydem.save_array_to_raster_extent(mydem.cppdem.get_DA_raster(), name = "DA", save_directory = "./")

		mydem.ExtractRiverNetwork(area_threshold_min = default_param["area_threshold"])
		# print(river.iloc[index_this])
		mydem.DefineCatchment( method="from_XY", X_coords = [river["X"][index_this]], Y_coords = [river["Y"][index_this]], coord_search_radius_nodes = 0)
		mydem.GenerateChi()
		print(mydem.df_base_river["source_key"].unique())

		mydem.cppdem.calculate_movern_disorder(0.05, 0.05, 19, 1, default_param["area_threshold"])
		all_disorder = mydem.cppdem.get_best_fits_movern_per_BK()
		global_results[str(round(river["flow_distance"][index_this],2))] = np.array(all_disorder[0]) # [0] means basin 0, which is normal as I only have one
		outlet_info["X"].append(river["X"][index_this])
		outlet_info["Y"].append(river["Y"][index_this])
		outlet_info["elevation"].append(river["elevation"][index_this])
		outlet_info["flow_distance"].append(round(river["flow_distance"][index_this],2))
		outlet_info["drainage_area"].append(river["drainage_area"][index_this])
		
		# New index
		flow_distance_this += default_param["flow_distance_step"]
		index_this = river.index[river["flow_distance"]>=flow_distance_this].values[0]

		np.savez("%sdown_to_top_conc.npz"%(default_param["prefix"]), **global_results)
		pd.DataFrame(outlet_info).to_csv("%sdown_to_top_conc_basin_info.csv"%(default_param["prefix"]), index = False)

		if(default_param["save_XY_array"]):
			this_basin = mydem.cppdem.query_xy_for_each_basin()[0] # [0] because I want a single basin there
			print(this_basin)
			XY_basins[str(round(river["flow_distance"][index_this],2))] = this_basin
			np.savez(default_param["prefix"] + "down_to_top_basins_XY.npz", **XY_basins)





def concavity_from_csv():
	"""
	FUnction to autoamte the extraction of concavity using a csv file to read the basin name and outlets.
	This is useful for very specific concavity extraction.
	B.G
	"""
	pass
	# COPYPASTED FROM OTHER FILE, NEED TO ADAPT
	# default_param = AGPD.get_common_default_param()
	# default_param["already_preprocessed"] = False
	# default_param["X_source"] = None
	# default_param["Y_source"] = None
	# default_param["min_elevation"] = None
	# default_param["area_threshold"] = None
	# default_param["flow_distance_step"] = None
	# default_param["min_DA"] = None
	# default_param["prefix"] = ""
	# default_param = AGPD.ingest_param(default_param, sys.argv)


	# if(default_param["help"] or len(sys.argv)==1):
	# 	print("""
	# 		This command-line tool run concavity analysis tools from LSDTopoTools.
	# 		Description of the algorithms in Mudd et al., 2018 -> https://www.earth-surf-dynam.net/6/505/2018/
	# 		To use, run the script with relevant options, for example:
	# 			lsdtt-concavity-down-to-top file=myraster.tif ---

	# 		option available:
	# 			file: name of the raster (file=name.tif)
	# 			path: path to the file (default = current folder)
	# 			todo
	# 			help: if written, diplay this message. Documentation soon to be written.
	# 		""")
	# 	quit()

	# try:
	# 	default_param["X_source"] = float(default_param["X_source"])
	# 	default_param["Y_source"] = float(default_param["Y_source"])
	# 	default_param["min_elevation"] = float(default_param["min_elevation"])
	# 	default_param["area_threshold"] = float(default_param["area_threshold"])
	# 	default_param["flow_distance_step"] = float(default_param["flow_distance_step"])
	# 	default_param["min_DA"] = float(default_param["min_DA"])
	# except:
	# 	print(default_param)
	# 	print("I struggle to understand your input parameters: make sure I can convert them to number")
	# 	raise SystemExit ("I need to quit now")	






"""
This file contains very basic but optimised routines for converting coordinates from WGS to UTM and reverse.
Highly adapted from https://github.com/Turbo87/utm, a lightweight utm packaged.
I numbaised it to ingest numpy array/ list of coordinates.
B.G.
"""
import numpy as np
import math
import numba as nb




################################################################################
################################################################################
# This is an optimization from https://github.com/Turbo87/utm
################################################################################
################################################################################


K0 = 0.9996

E = 0.00669438
E2 = E * E
E3 = E2 * E
E_P2 = E / (1.0 - E)

SQRT_E = math.sqrt(1 - E)
_E = (1 - SQRT_E) / (1 + SQRT_E)
_E2 = _E * _E
_E3 = _E2 * _E
_E4 = _E3 * _E
_E5 = _E4 * _E

M1 = (1 - E / 4 - 3 * E2 / 64 - 5 * E3 / 256)
M2 = (3 * E / 8 + 3 * E2 / 32 + 45 * E3 / 1024)
M3 = (15 * E2 / 256 + 45 * E3 / 1024)
M4 = (35 * E3 / 3072)

P2 = (3. / 2 * _E - 27. / 32 * _E3 + 269. / 512 * _E5)
P3 = (21. / 16 * _E2 - 55. / 32 * _E4)
P4 = (151. / 96 * _E3 - 417. / 128 * _E5)
P5 = (1097. / 512 * _E4)

R = 6378137

ZONE_LETTERS = "CDEFGHJKLMNPQRSTUVWXX"


def to_latlon(easting, northing, zone_number, northern=None):
  """
    Main function to convert UTM to latitude-longitude.
    Takes XY and returns latlon basically.
    Arguments:
      easting (float or array): list of easting (X) coordinates
      northing (float or array): list of northing (Y) coordinates
      zone_number: the inputed UTM zone -> http://www.jaworski.ca/utmzones.htm
      northern (Boolean): are you in the northern hemisphere?
    returns:
      Latitude, longitude arrays
    Authors:
      B.G.
    Date:
      At some points in 2018

  """


  if(isinstance(easting,list)):
    easting = np.asarray(easting)
    northing = np.asarray(northing)

  if(isinstance(easting,np.ndarray)):
    return _to_latlon(easting, northing, zone_number,  northern)
  else:
    return _to_latlon([easting], [northing], zone_number,  northern)


@nb.jit(nopython = False)
def _to_latlon(easting, northing, zone_number, northern):

    """Internal function convert an UTM coordinate into Latitude and Longitude
        Parameters
        ----------
        easting: int
            Easting value of UTM coordinate
        northing: int
            Northing value of UTM coordinate
        zone number: int
            Zone Number is represented with global map numbers of an UTM Zone
            Numbers Map. More information see utmzones [1]_
        zone_letter: str
            Zone Letter can be represented as string values. Where UTM Zone
            Designators can be accessed in [1]_
        northern: bool
            You can set True or False to set this parameter. Default is None
       .. _[1]: http://www.jaworski.ca/utmzones.htm
    """

    longitude = []
    latitude = []

    for i in range(easting.shape[0]):

      x = easting[i] - 500000
      y = northing[i]

      if not northern:
          y -= 10000000

      m = y / K0
      mu = m / (R * M1)

      p_rad = (mu +
               P2 * math.sin(2 * mu) +
               P3 * math.sin(4 * mu) +
               P4 * math.sin(6 * mu) +
               P5 * math.sin(8 * mu))

      p_sin = math.sin(p_rad)
      p_sin2 = p_sin * p_sin

      p_cos = math.cos(p_rad)

      p_tan = p_sin / p_cos
      p_tan2 = p_tan * p_tan
      p_tan4 = p_tan2 * p_tan2

      ep_sin = 1 - E * p_sin2
      ep_sin_sqrt = math.sqrt(1 - E * p_sin2)

      n = R / ep_sin_sqrt
      r = (1 - E) / ep_sin

      c = _E * p_cos**2
      c2 = c * c

      d = x / (n * K0)
      d2 = d * d
      d3 = d2 * d
      d4 = d3 * d
      d5 = d4 * d
      d6 = d5 * d

      latitude.append(math.degrees(p_rad - (p_tan / r) *
                        (d2 / 2 -
                         d4 / 24 * (5 + 3 * p_tan2 + 10 * c - 4 * c2 - 9 * E_P2)) +
                         d6 / 720 * (61 + 90 * p_tan2 + 298 * c + 45 * p_tan4 - 252 * E_P2 - 3 * c2)))

      longitude.append(math.degrees(d -
                         d3 / 6 * (1 + 2 * p_tan2 + c) +
                         d5 / 120 * (5 - 2 * c + 28 * p_tan2 - 3 * c2 + 8 * E_P2 + 24 * p_tan4) / p_cos ) + (zone_number - 1) * 6 - 180 + 3)

    return (latitude,longitude)


def from_latlon(latitude, longitude, force_zone_number= -9999):
  """
    Highest level function to convert latitude longitude (WGS84) to UTM.
    Arguments:
      latitude (scalar or list or array): inputted latitude in WGS84
      longitude (scalar or list or array): inputted longitude in WGS84
      force_zone_number (scalar): Zone number will be determined automatically, but you can eventually force one.
    Returns:
      easting,northing (X,Y) coordinate arrays
    Authors:
      B.G. (again from https://github.com/Turbo87/utm)
    Date:
      2018, summer, I think.
  """

  if(isinstance(longitude,list)):
    longitude = np.asarray(longitude)
    latitude = np.asarray(latitude)

  if(isinstance(longitude,np.ndarray)):
    return _from_latlon(latitude, longitude, force_zone_number)
  else:
    return _from_latlon([latitude], [longitude], force_zone_number)



@nb.jit(nopython = True, parallel = True)
def _from_latlon(latitude, longitude, force_zone_number):
    """This function convert Latitude and Longitude to UTM coordinate
        Parameters
        ----------
        latitude: float
            Latitude between 80 deg S and 84 deg N, e.g. (-80.0 to 84.0)
        longitude: float
            Longitude between 180 deg W and 180 deg E, e.g. (-180.0 to 180.0).
        force_zone number: int
            Zone Number is represented with global map numbers of an UTM Zone
            Numbers Map. You may force conversion including one UTM Zone Number.
            More information see utmzones [1]_
       .. _[1]: http://www.jaworski.ca/utmzones.htm
    """
    easting = []
    northing = []

    for i in range(longitude.shape[0]):
      lat_rad = math.radians(latitude[i])
      lat_sin = math.sin(lat_rad)
      lat_cos = math.cos(lat_rad)

      lat_tan = lat_sin / lat_cos
      lat_tan2 = lat_tan * lat_tan
      lat_tan4 = lat_tan2 * lat_tan2

      if(force_zone_number < 0):
        zone_number = latlon_to_zone_number(latitude[i], longitude[i])

      else:
        zone_number = force_zone_number

      lon_rad = math.radians(longitude[i])
      central_lon = (zone_number - 1) * 6 - 180 + 3
      central_lon_rad = math.radians(central_lon)

      n = R / math.sqrt(1 - E * lat_sin**2)
      c = E_P2 * lat_cos**2

      a = lat_cos * (lon_rad - central_lon_rad)
      a2 = a * a
      a3 = a2 * a
      a4 = a3 * a
      a5 = a4 * a
      a6 = a5 * a

      m = R * (M1 * lat_rad -
               M2 * math.sin(2 * lat_rad) +
               M3 * math.sin(4 * lat_rad) -
               M4 * math.sin(6 * lat_rad))

      easting.append(K0 * n * (a +
                          a3 / 6 * (1 - lat_tan2 + c) +
                          a5 / 120 * (5 - 18 * lat_tan2 + lat_tan4 + 72 * c - 58 * E_P2)) + 500000)

      northing.append(K0 * (m + n * lat_tan * (a2 / 2 +
                                          a4 / 24 * (5 - lat_tan2 + 9 * c + 4 * c**2) +
                                          a6 / 720 * (61 - 58 * lat_tan2 + lat_tan4 + 600 * c - 330 * E_P2))))

      if latitude[i] < 0:
          northing[-1] += 10000000

    return easting, northing, zone_number


# def latitude_to_zone_letter(latitude):
#     if -80 <= latitude <= 84:
#         return ZONE_LETTERS[int(latitude + 80) >> 3]
#     else:
#         return None

@nb.jit(nopython = True, cache = True)
def latlon_to_zone_number(latitude, longitude):
    if 56 <= latitude < 64 and 3 <= longitude < 12:
        return 32

    if 72 <= latitude <= 84 and longitude >= 0:
        if longitude < 9:
            return 31
        elif longitude < 21:
            return 33
        elif longitude < 33:
            return 35
        elif longitude < 42:
            return 37

    return int((longitude + 180) / 6) + 1


"""
__innit__ for lsdtopytools. That's wher you have to add your new modules if you create new.
Contact B.G. for info. 
"""

__author__ = """Boris Gailleton"""
__email__ = 'b.gailleton@sms.ed.ac.uk'
__version__ = '0.0.1'
import sys
# Dealing with the imports: making it compatible with python 3 and 2
if(sys.version[0] == 2):
	from raster_loader import *
	from geoconvtools import *
	from lsdtopytools_utilities import *
	from lsdtopytools import LSDDEM
	# from LSDTribBas import *
	from quickplot import *
	from quickplot_utilities import *
	from quickplot_movern import *
	from quickplot_ksn_knickpoints import *
	from argparser_debug import *
	# from Muddpyle import *
	from numba_tools import *
	from concavity_automator import *
else:
	from .raster_loader import *
	from .geoconvtools import *
	from .lsdtopytools_utilities import *
	from .lsdtopytools import LSDDEM
	# from .LSDTribBas import *
	from .quickplot import *
	from .quickplot_utilities import *
	from .quickplot_movern import *
	from .quickplot_ksn_knickpoints import *
	from .argparser_debug import *
	# from .Muddpyle import *
	from .numba_tools import *
	from .concavity_automator import *

	try:
		from .shapefile_tools import *
	except ImportError:
		shp_not_installed = True
	# try:
	# 	from .fastscapelib_ultimate_binding_lsdtt import *
	# except ImportError:
	# 	shp_not_installed = True


"""
These tools are probably there because I needed very quick test of relatively computationally expensive shits that will get ported to cpp when I'll have time to actually compile stuffs.
best,
Boris
"""

import numpy as np
import numba as nb
import math
M_PI = np.pi

# @nb.jit(nopython = True)
# def thin_perimeter_for_salesman(X,Y,distance_thinning):

#   new_index = np.zeros(X.shape[0])


#   return new_index

@nb.jit(nopython = True)
def travelling_salesman_algortihm(X,Y):

  is_visited = np.zeros(X.shape[0])
  temp_distance = np.zeros(X.shape[0])
  new_index = np.zeros(X.shape[0], dtype = np.int32)

  new_index[0] = 0
  is_visited[0] = 1
  that_index = 0

  for i in range(1,new_index.shape[0]):

    that_x = X[that_index]
    that_y = Y[that_index]

    min_distance = 1e32
    min_id = -9999

    #compute distance
    temp_distance[that_index] = min_distance
    for j in range(new_index.shape[0]):
      if(j!= that_index):
        temp_distance[j] = math.sqrt(abs(X[j] - that_x)**2 + abs(Y[j] - that_y) **2)
        if(temp_distance[j] < min_distance and is_visited[j] == 0):
          min_distance = temp_distance[j]
          min_id = j

    new_index[i] = min_id
    that_index = min_id
    is_visited[that_index] = 1

  return new_index

@nb.jit(nopython = True)
def remove_outliers_in_drainage_divide(Dx, Dy, threshold):

  indices = np.full(Dx.shape[0], True)

  is_outlier = False
  last = 0

  for i in range(Dx.shape[0]):
    if(Dx[i]>threshold and Dy[i]>threshold):
      is_outlier = True

    if(is_outlier):
      indices[i] = False

  return indices



@nb.jit(nopython=False, parallel = True)
def average_from_grid_n_stuff_maybe_adaptative_and_all(X,Y,Z,window):
	"""
	Take X,Y,Z and a window size. Return an array of resampled stuffs from that window size.
	Not sure how it exacty works
	B.G.
	"""

	# First getting the minimum and maximum
	Xminimum = np.min(X)
	Yminimum = np.min(Y)
	Xmaximum = np.max(X)
	Ymaximum = np.max(Y)
	# print(Xminimum)
	# print(Yminimum)
	# print(Xmaximum)
	# print(Ymaximum)

	n_cols = np.int32(round((Xmaximum-Xminimum)/window))
	n_rows = np.int32(round((Ymaximum-Yminimum)/window))
	
	avevarray = np.zeros((n_rows,n_cols))
	n = np.zeros((n_rows,n_cols))

	X_coord = np.zeros(n_cols)
	Y_coord = np.zeros(n_rows)

	dat_X = Xminimum
	for i in range(n_cols):
		# print(dat_X)
		X_coord[i] = dat_X + (window)/2
		dat_X += window

	dat_Y = Yminimum
	for i in range(n_rows):
		Y_coord[i] = dat_Y + (window)/2
		dat_Y += window
	# Inverting Y coordinate
	Y_coord = Y_coord[::-1]

	helfwind = window/2
	for i in nb.prange(n_rows):
		for j in range(n_cols):
			# print("BITE")
			if(Z[((X<X_coord[j]+helfwind) & (X>=X_coord[j]-helfwind) & (Y < Y_coord[i]+helfwind) & (Y >= Y_coord[i]-helfwind) ) ].shape[0]>0):
				avevarray[i,j] = np.median(Z[((X<X_coord[j]+helfwind) & (X>=X_coord[j]-helfwind) & (Y < Y_coord[i]+helfwind) & (Y >= Y_coord[i]-helfwind) ) ] )

	return avevarray, X_coord, Y_coord



def numba_hillshading(arr,res,azimuth = 315,angle_altitude = 45,z_factor = 1., parallel = True, ignore_edge = False):
  """
  Numba mnplementation of the hillshading algorithm present in LSDTT c++ algorithmath.
  This is basically just to train myself before optimization/parrallelization with numba
  BG, from DAV-SWDG-SMM codes in LSDTopoTools
  """

  HSarray = np.empty(arr.shape, dtype = np.float32)
  HSarray.fill(-9999)

  zenith_rad = (90 - angle_altitude) * M_PI / 180.0
  azimuth_math = 360-azimuth + 90

  if (azimuth_math >= 360.0):
    azimuth_math = azimuth_math - 360
  azimuth_rad = azimuth_math * M_PI  / 180.0

  slope_rad = 0
  aspect_rad = 0
  dzdx = 0
  dzdy = 0
  i = 0
  j = 0
  nrows = arr.shape[0] -1
  ncols = arr.shape[1] -1

  if(ignore_edge):
    arr[:,0] = -9999
    arr[:,-1] = -9999

    arr[0,:] = -9999
    arr[-1,:] = -9999
    

  # print("I am hillshading")

  if(not parallel):
    return _hillshading(arr,res,ncols,nrows,HSarray,zenith_rad,azimuth_rad, z_factor)
  elif(parallel):
    return _parallel_hillshading(arr,res,ncols,nrows,HSarray,zenith_rad,azimuth_rad, z_factor)

@nb.jit(nopython = True,cache = True)
def _hillshading(arr,res,ncols,nrows,HSarray,zenith_rad,azimuth_rad, z_factor):
  """
  Slow and basic imnplementation of the hillshading algorithm present in LSDTT c++ algorithmath.
  This is basically just to train myself before optimization/parrallelization with numba
  BG, from DAV-SWDG-SMM codes
  """

  for i in range(nrows):
    for j in range(ncols):

      if arr[i, j] != -9999:

        dzdx = (((arr[i, j+1] + 2*arr[i+1, j] + arr[i+1, j+1]) -
                (arr[i-1, j-1] + 2*arr[i-1, j] + arr[i-1, j+1]))
                / (8 * res))
        dzdy = (((arr[i-1, j+1] + 2*arr[i, j+1] + arr[i+1, j+1]) -
                (arr[i-1, j-1] + 2*arr[i, j-1] + arr[i+1, j-1]))
                / (8 * res))

        slope_rad = np.arctan(z_factor * np.sqrt((dzdx * dzdx) + (dzdy * dzdy)))

        if (dzdx != 0):
          aspect_rad = np.arctan2(dzdy, (dzdx*-1))
          if (aspect_rad < 0):
            aspect_rad = 2 * M_PI + aspect_rad

        else:
          if (dzdy > 0):
            aspect_rad = M_PI / 2
          elif (dzdy < 0):
            aspect_rad = 2 * M_PI - M_PI / 2
          else:
            aspect_rad = aspect_rad

        HSarray[i, j] = 255.0 * ((np.cos(zenith_rad) * np.cos(slope_rad)) +
                        (np.sin(zenith_rad) * np.sin(slope_rad) *
                        np.cos(azimuth_rad - aspect_rad)))
        # Necessary?
        if (HSarray[i, j] < 0):
          HSarray[i, j] = 0
  
  return HSarray 

@nb.jit(nopython = True, parallel = True)
def _parallel_hillshading(arr,res,ncols,nrows,HSarray,zenith_rad,azimuth_rad, z_factor):
  """
  Slow and basic imnplementation of the hillshading algorithm present in LSDTT c++ algorithmath.
  This is basically just to train myself before optimization/parrallelization with numba
  BG, from DAV-SWDG-SMM codes
  """

  for i in nb.prange(nrows):
    for j in range(ncols):

      if arr[i, j] != -9999:

        dzdx = (((arr[i, j+1] + 2*arr[i+1, j] + arr[i+1, j+1]) -
                (arr[i-1, j-1] + 2*arr[i-1, j] + arr[i-1, j+1]))
                / (8 * res))
        dzdy = (((arr[i-1, j+1] + 2*arr[i, j+1] + arr[i+1, j+1]) -
                (arr[i-1, j-1] + 2*arr[i, j-1] + arr[i+1, j-1]))
                / (8 * res))

        slope_rad = np.arctan(z_factor * np.sqrt((dzdx * dzdx) + (dzdy * dzdy)))

        if (dzdx != 0):
          aspect_rad = np.arctan2(dzdy, (dzdx*-1))
          if (aspect_rad < 0):
            aspect_rad = 2 * M_PI + aspect_rad

        else:
          if (dzdy > 0):
            aspect_rad = M_PI / 2
          elif (dzdy < 0):
            aspect_rad = 2 * M_PI - M_PI / 2
          else:
            aspect_rad = aspect_rad

        HSarray[i, j] = 255.0 * ((np.cos(zenith_rad) * np.cos(slope_rad)) +
                        (np.sin(zenith_rad) * np.sin(slope_rad) *
                        np.cos(azimuth_rad - aspect_rad)))
        # Necessary?
        if (HSarray[i, j] < 0):
          HSarray[i, j] = 0
  
  return HSarray


#!/usr/bin/env python
"""
Command-line tool to control the concavity constraining tools
Mudd et al., 2018
So far mostly testing purposes
B.G.
"""
from lsdtopytools import LSDDEM, raster_loader as rl # I am telling python I will need this module to run.
from lsdtopytools import argparser_debug as AGPD # I am telling python I will need this module to run.
from lsdtopytools import quickplot as qp, quickplot_movern as qmn # We will need the plotting routines
import numpy as np
from matplotlib import pyplot as plt
from lsdtopytools import quickplot_utilities as QU
import time as clock # Basic benchmarking
import sys # manage the argv
import pandas as pd

def PreProcess():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["breach"] = False
	default_param["fill"] = False
	default_param["min_slope"] = 0.0001
	default_param["save_path"] = "./"

	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool provides basic Preprocessing functions for a raster.
			Breaching is achieved running https://doi.org/10.1002/hyp.10648 algorithm (Lindsay2016), with an implementation from RICHDEM
			Filling is from Wang and liu 2006. I think MDH implemented the code.
			It also "clean" the raster and tidy the nodata in it.
			To use, run the script with relevant options, for example:
			-> filling and breaching
				lsdtt-depressions file=myraster.tif fill breach 0.00001 

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				breach: want to breach?
				fill: want to fill?
				min_slope: imply a min slope for filling

			""")
		quit()

	sevadir = default_param["save_path"]
	
	print("Welcome to the command-line tool to .")
	print("Let me first load the raster ...")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = False, verbose = False)
	print("Got it. Now dealing with the depressions ...")
	mydem.PreProcessing(filling = default_param["fill"], carving = default_param["breach"], minimum_slope_for_filling = float(default_param["min_slope"])) # Unecessary if already preprocessed of course.

	mydem.save_dir

	print("Saving the raster! same name basically, but with _PP at the end")
	rl.save_raster(mydem.cppdem.get_PP_raster(),mydem.x_min,mydem.x_max,mydem.y_max,mydem.y_min,mydem.resolution,mydem.crs,mydem.path+sevadir+mydem.prefix + "_PP.tif", fmt = 'GTIFF')

def Polyfit_Metrics():

	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["window_radius"] = 30

	default_param["average_elevation"] = False
	default_param["slope"] = False
	default_param["aspect"] = False
	default_param["curvature"] = False
	default_param["planform_curvature"] = False
	default_param["profile_curvature"] = False
	default_param["tangential_curvature"] = False
	default_param["TSP"] = False



	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			Get the polyfit metrics: average_elevation, slope, aspect, curvature, planform_curvature, profile_curvature, tangential_curvature
			by fitting a plane equation for each pixels through the neighboring node within a distance.
			
			Quick example: lsdtt-polyfits file=myraster.tif slope window_radius=17
			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				average_elevation(add keyword to activate): get the average_elevation of the polyfit raster
				slope(add keyword to activate): get the slope of the polyfit raster
				aspect(add keyword to activate): get the aspect of the polyfit raster
				curvature(add keyword to activate): get the curvature of the polyfit raster
				planform_curvature(add keyword to activate): get the planform_curvature of the polyfit raster
				profile_curvature(add keyword to activate): get the profile_curvature of the polyfit raster
				tangential_curvature(add keyword to activate): get the tangential_curvature of the polyfit raster
				window_radius: distance around each pixels to fit the plane

			Future improvements:
			- Automatic plotting option
			- others?
			""")
		quit()


	print("Welcome to the command-line tool to .")
	print("Let me first load the raster ...")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = True, verbose = False)

	res = mydem.get_polyfit_metrics(window_radius = default_param["window_radius"], average_elevation = default_param["average_elevation"], slope = default_param["slope"], aspect = default_param["aspect"], curvature = default_param["curvature"], planform_curvature = default_param["planform_curvature"], profile_curvature = default_param["profile_curvature"], tangential_curvature = default_param["tangential_curvature"], TSP = default_param["TSP"], save_to_rast = True)

	if(default_param["slope"]):
		fig,ax = plt.subplots()
		cb =ax.imshow(res['slope'], extent = mydem.extent, vmin = np.percentile(res['slope'], 20), vmax = np.percentile(res['slope'], 80) )
		QU.fix_map_axis_to_kms(ax, 8,4)
		plt.colorbar(cb)
		plt.savefig(mydem.path+mydem.prefix + "_slope_%s.png"%(default_param["window_radius"]), dpi = 500)




def topomap():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["save_hillshade"] = True
	default_param["topomap"] = True

	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool provides first order topographic visualisation:

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				save_hillshade: save a tif hillshade file
				topomap: save a nice topographic map

			""")
		quit()


	print("Welcome to the command-line tool to .")
	print("Let me first load the raster ...")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = True, verbose = False)

	if(default_param["topomap"]):
		qp.plot_nice_topography(mydem ,figure_width = 6, figure_width_units = "inches", cmap = "gist_earth", hillshade = True, 
	alpha_hillshade = 0.45, color_min = None, color_max = None, dpi = 500, output = "save", format_figure = "png", fontsize_ticks =6, fontsize_label = 8, 
	hillshade_cmin = 0, hillshade_cmax = 250,colorbar = True, 
	fig = None, ax = None, colorbar_label = None, colorbar_ax = None)

	if(default_param["save_hillshade"]):
		print("Saving the Hillshade raster ...")
		rl.save_raster(mydem.get_hillshade(),mydem.x_min,mydem.x_max,mydem.y_max,mydem.y_min,mydem.resolution,mydem.crs,mydem.path+mydem.prefix + "_hs.tif", fmt = 'GTIFF')

def remove_seas():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["sea_level"] = 0
	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool preprocess your raster to the right format and remove elevation below a threshold, typically the sea.

			option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				sea_level: elevation threshold (default = 0)

			""")
		quit()


	print("Loading the dem")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = True, verbose = False)
	print("Processing the sea, removing everything %s ..."%(float(default_param["sea_level"])))
	topo = np.copy(mydem.cppdem.get_PP_raster())
	topo[ topo < float(default_param["sea_level"]) ] = -9999
	print("done, let me save the raster")
	rl.save_raster(topo,mydem.x_min,mydem.x_max,mydem.y_max,mydem.y_min,mydem.resolution,mydem.crs,mydem.path+mydem.prefix + "_sea_removed.tif", fmt = 'GTIFF')


def extract_single_river_from_source():
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["already_preprocessed"] = False
	default_param["X"] = 0
	default_param["Y"] = 0
	default_param["output_name"] = "Single_River"
	default_param = AGPD.ingest_param(default_param, sys.argv)

	if(default_param["help"] or len(sys.argv)==1):
		print("""
			This command-line tool preprocess your raster to the right format and remove elevation below a threshold, typically the sea.

			Quick example: lsdtt-extract-single-river file=test_raster.tif X=592149.7 Y=4103817.2

			Option available:
				file: name of the raster (file=name.tif)
				path: path to the file (default = current folder)
				already_preprocessed (default = False): Add the keyword if your raster is already preprocessed for flow analysis, False if it needs preprocessing. To get more control on the preprocessing you can use lsdtt-depressions command.
				X: Easting/X coordinate of the source
				Y: Easting/Y coordinate of the source
				output_name (default = Single_River): prefix of the ouput(s) files.

			Future improvements:
				- Automatic plotting of base statistic
				- Adding Chi/ksn/gradient options
			""")
		quit()

	print("This command-line tool extract a single river from a DEM from the XY coordinates of the source")
	print("Loading the dem")
	mydem = LSDDEM(file_name = default_param["file"], path=default_param["path"], already_preprocessed = bool(default_param["already_preprocessed"]), verbose = True)
	if(bool(default_param["already_preprocessed"]) == False):
		print("I am preprocessing your dem (carving + filling), if you have already done that, you can save time by adding the option already_preprocessed=True to the command line (it saves time)")
		mydem.PreProcessing()

	river = pd.DataFrame(mydem.ExtractSingleRiverFromSource(float(default_param["X"]),float(default_param["Y"])))
	river.to_csv("%s.csv"%(default_param["output_name"]), index = False)


# def basinator():
	


"""
This class deals with loading raster informations
Authors: B.G.
"""
import numpy as np
import rasterio as rio
from rasterio.transform import from_bounds


def load_raster(fname):
	"""
	Load a raster array with different options. It uses rasterio that itself uses gdal.
	Arguments:
		fname (str): raster to load (path+file_name+format)
	Returns:
		A python dictionnary containing the following "key" -> val:
			"res" -> Resolution of the DEM
			"ncols" -> number of columns
			"nrows" -> number of rows
			"x_min" -> well x minimum
			"y_min" -> and y minimum
			"x_max" -> and x maximum
			"y_max" -> and x maximum
			"extent" -> extent combined in order to match matplotlib
			"array" -> numpy 2D array containing the data
			"crs" -> The crs string (geolocalisation)
			"nodata" -> list of nodata values
	Authors:
		B.G.
	Date:
		23/02/2019
	"""

	# Loading the raster with rasterio
	this_raster = rio.open(fname)

	# Initialising a dictionary containing the raster info for output
	out = {}
	# I am padding a no_data contour
	gt = this_raster.res
	out['res'] = gt[0]
	out["ncols"] = this_raster.width
	out["nrows"] = this_raster.height
	out["x_min"] = this_raster.bounds[0]
	out["y_min"] = this_raster.bounds[1]
	out["x_max"] = this_raster.bounds[2]
	out["y_max"] = this_raster.bounds[3]
	corr = out['res']*2
	out["extent"] = [out["x_min"],out["x_max"]-corr,out["y_min"],out["y_max"]-corr]
	out["array"] = this_raster.read(1)
	try:
		out['crs'] = this_raster.crs['init']
	except (TypeError, KeyError) as e:
		out['crs'] = u'epsg:32601'
	out['nodata'] = this_raster.nodatavals

	
	
	# pixelSizeY =-gt[4]
	# (left=358485.0, bottom=4028985.0, right=590415.0, top=4265115.0)

	return out

def save_raster(Z,x_min,x_max,y_min,y_max,res,crs,fname, fmt = 'GTIFF'):
	'''
		Save a raster at fname. I followed tutorial there to do so : https://rasterio.readthedocs.io/en/latest/quickstart.html#creating-data
		Arguments:
			Z (2d numpy array): the array
			x_min (float): x min
			y_min (float): y min
			x_max (float): x max
			y_max (float): y max
			res (float): resolution
			crs: coordinate system code (usually you will call it from a dem object, just give it dem.crs)
			fname: path+name+.tif according to your OS  ex: Windows: C://Data/Albania/Holtas/Holt.tif, Linux: /home/Henri/Albania/Shenalvash/She_zoom.tif
			fmt (str): string defining the format. "GTIFF" for tif files, see GDAL for option. WARNING: few outputs are buggy, FOR EXAMPLE "ENVI" bil format can be with the wrong dimensions.
		Returns:
			Nothing but saves a raster with the given parameters
		Authors: 
			B.G. 
		Date:
			08/2018
	'''

	transform = from_bounds(x_min, y_max, x_max, y_min, Z.shape[1], Z.shape[0])
	new_dataset = rio.open(fname, 'w', driver=fmt, height=Z.shape[0], width=Z.shape[1], count=1, dtype=Z.dtype.type, crs=crs, transform=transform, nodata = -9999)
	new_dataset.write(Z, 1)
	new_dataset.close()


from tkinter import *
from tkinter import PhotoImage, Canvas
import numpy as np
import pandas as pd
from PIL import Image, ImageTk
from matplotlib import cm

class Window(Frame):
  def __init__(self, master=None, width = 400, height = 300):
    Frame.__init__(self, master)               
    self.master = master
    self.width = width
    self.height = height
    self.init_window()
    # self.canvas = Canvas(self, width=width, height=height, bg="#000000")
    # self.canvas.pack(expand=YES, fill=BOTH)


  def init_window(self, title = "LSDTopoTools"):

    self.master.title(title)
    self.pack(fill=BOTH, expand=1)
    self.master.geometry("%sx%s"%(self.width,self.height))

  def plot_array(self, array):
    # im = Image.fromarray(np.array([[20,50,50,20],[35,35,250,2]]))
    # newarr = np.resize(array,(self.height,self.width))
    im = Image.fromarray(np.uint8(cm.gist_earth(array)*255))
    im.save("test.png")
    img = ImageTk.PhotoImage(im)
    # print(img)
    label = Label(image=img) 
    label.image = img # keep a reference!
    label.pack(expand = YES, fill=BOTH)





"""
Command-line tool to control the concavity constraining tools
Mudd et al., 2018
So far mostly testing purposes
B.G.
"""
from lsdtopytools import LSDDEM, raster_loader as rl # I am telling python I will need this module to run.
from lsdtopytools import argparser_debug as AGPD # I am telling python I will need this module to run.
from lsdtopytools import quickplot as qp, quickplot_movern as qmn # We will need the plotting routines
import numpy as np
from matplotlib import pyplot as plt
from lsdtopytools import quickplot_utilities as QU
import time as clock # Basic benchmarking
import sys # manage the argv
import glob
import multiprocessing
import subprocess
# from itertools import product

def load_n_plot(name):
	MD = LSDDEM(file_name = name, already_preprocessed = True)
	qp.plot_nice_topography(MD ,figure_width = 4, figure_width_units = "inches", cmap = "gist_earth", hillshade = True, 
alpha_hillshade = 0.45, color_min = None, color_max = None, dpi = 300, output = "save", format_figure = "png", fontsize_ticks =6, fontsize_label = 8, 
hillshade_cmin = 0, hillshade_cmax = 250,colorbar = False, 
fig = None, ax = None, colorbar_label = None, colorbar_ax = None, force_path = True, path_to_force = "./")
	term = MD.path+MD.prefix+"/"
	subprocess.run("rm -r %s"%(term), shell = True)

def plot_all_tif_of_folder():


	
	# Here are the different parameters and their default value fr this script
	default_param = AGPD.get_common_default_param()
	default_param["n_threads"] = 4
	default_param = AGPD.ingest_param(default_param, sys.argv)

	file_to_process = []
	suffix = []
	cpt = 1
	for file in glob.glob("*.tif"):
		file_to_process.append( (file,) )
		name = str(cpt)
		cpt+=1
	for file in glob.glob("*.tiff"):
		file_to_process.append( (file,) )
	if(len(file_to_process)==0):
		print("I did not find any file! Aborting.")
		quit()

	with multiprocessing.Pool(processes=int(default_param["n_threads"])) as pool:
		pool.starmap(load_n_plot, file_to_process)

	print("Done with all the rasters")